#import <Foundation/Foundation.h>
#import <oonimkall/Oonimkall.h>
#import "OONILogger.h"

/** OONISessionConfig contains configuration for a OONISession. */
@interface OONISessionConfig : NSObject
/**
 * assetsDir is a mandatory setting specifying the directory in
 * which you want the OONI Probe engine to store the assets.
 */
@property (nonatomic, strong) NSString* assetsDir;

/**
 * logger is an optional setting specifying a logger where
 * to send log messages generated by OONI Probe's engine.
 */
@property (nonatomic, strong) id<OONILogger> logger;

/**
 * ProbeServicesURL allows you to optionally force the
 * usage of an alternative probe services. This setting
 * should only be used to implement integration tests.
 */
@property (nonatomic, strong) NSString*  probeServicesURL;

/**
 * softwareName is a mandatory setting specifying the name of
 * the application that is using OONI Probe's engine.
 */
@property (nonatomic, strong) NSString*  softwareName;

/**
 * softwareVersion is a mandatory setting specifying the version of
 * the application that is using OONI Probe's engine.
 */
@property (nonatomic, strong) NSString*  softwareVersion;

/**
 * stateDir is a mandatory setting specifying the directory in
 * which you want the OONI Probe engine to store its state.
 */
@property (nonatomic, strong) NSString*  stateDir;

/**
 * tempDir is a mandatory setting specifying the directory in which
 * you want the OONI Probe engine to store temporary files.
 */
@property (nonatomic, strong) NSString*  tempDir;

/**
 * verbose controls whether to emit debug messages. This setting only
 * has effect if you also specify a Logger using setLogger.
 */
@property (nonatomic) BOOL verbose;

- (OonimkallSessionConfig*) toOonimkallSessionConfig;

@end
